/*
 * Copyright 2002-2003. The Regents of the University of California. This material 
 * was produced under U.S. Government contract W-7405-ENG-36 for Los Alamos 
 * National Laboratory, which is operated by the University of California for 
 * the U.S. Department of Energy. The Government is granted for itself and 
 * others acting on its behalf a paid-up, nonexclusive, irrevocable worldwide 
 * license in this material to reproduce, prepare derivative works, and 
 * perform publicly and display publicly. Beginning five (5) years after 
 * October 10,2002 subject to additional five-year worldwide renewals, the 
 * Government is granted for itself and others acting on its behalf a paid-up, 
 * nonexclusive, irrevocable worldwide license in this material to reproduce, 
 * prepare derivative works, distribute copies to the public, perform publicly 
 * and display publicly, and to permit others to do so. NEITHER THE UNITED 
 * STATES NOR THE UNITED STATES DEPARTMENT OF ENERGY, NOR THE UNIVERSITY OF 
 * CALIFORNIA, NOR ANY OF THEIR EMPLOYEES, MAKES ANY WARRANTY, EXPRESS OR 
 * IMPLIED, OR ASSUMES ANY LEGAL LIABILITY OR RESPONSIBILITY FOR THE ACCURACY, 
 * COMPLETENESS, OR USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT, OR 
 * PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE PRIVATELY 
 * OWNED RIGHTS.

 * Additionally, this program is free software; you can distribute it and/or 
 * modify it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2 of the License, 
 * or any later version.  Accordingly, this program is distributed in the hope 
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU Lesser General Public License for more details.
 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include "ulm/ulm.h"
#include "internal/log.h"
#include "internal/profiler.h"
#include "internal/types.h"
#include "queue/globals.h"

extern "C" int ulm_scatterv_p2p(void *sendbuf, int *sendcounts, int *displs,
                                ULMType_t *sendtype, void *recvbuf,
                                int recvcount, ULMType_t *recvtype, int root,
                                int comm)
{
    int proc, nproc, self, rc, tag, completed;
    ULMRequest_t rreq, *sreq;
    ULMStatus_t status;
    unsigned char *ptr;

    ulm_get_info(comm, ULM_INFO_PROCID, &self, sizeof(int));
    ulm_get_info(comm, ULM_INFO_NUMBER_OF_PROCS, &nproc, sizeof(int));
    rc = ulm_get_system_tag(comm, 1, &tag);
    if (rc != ULM_SUCCESS) {
	return MPI_ERR_INTERN;
    }

    if (self == root) {
        /* first, post a receive from myself */
        rc = ulm_irecv(recvbuf, recvcount, recvtype, root, tag, comm, &rreq);
        if (rc != ULM_SUCCESS) {
            return rc;
        }

        /* allocate array of send request handles */
        sreq = (ULMRequest_t *) 
            malloc(nproc * sizeof(ULMRequest_t));
        if (sreq == NULL) {
            return ULM_ERROR;
        }

        /* send to everyone, including myself */
        for (proc = 0; proc < nproc; proc++) { 
            ptr = (unsigned char *) sendbuf + displs[proc] * sendtype->extent;
            rc = ulm_isend(ptr, sendcounts[proc], sendtype, proc, 
                           tag, comm, &sreq[proc], ULM_SEND_STANDARD);
            if (rc != ULM_SUCCESS) {
                return rc;
            }
        }

        /* wait on the sends */
        completed = 0;
        while (!completed) {
            rc = ulm_testall(sreq, nproc, &completed);
            if (rc != ULM_SUCCESS) {
                return rc;
            }
        }

        /* wait on the recv from myself */
        rc = ulm_wait(&rreq, &status);
        if (rc != ULM_SUCCESS) {
            return rc;
        }

        free(sreq), sreq = 0;
    } else {
        rc = ulm_irecv(recvbuf, recvcount, recvtype, root, tag, comm, &rreq);
        if (rc != ULM_SUCCESS) {
            return rc;
        }
        rc = ulm_wait(&rreq, &status);
        if (rc != ULM_SUCCESS) {
            return rc;
        }
    }

    return ULM_SUCCESS;
}
